@@ -2,19 +2,18 @@ discoveries <- function(hommel, ix, incremental=FALSE, alpha=0.05)
 {
   m <- length(hommel@p)
   if (missing(ix) & incremental==FALSE) {
-    p = hommel@p
-    k = m
+    k  <- m
+    ix <- hommel@sorter
   } 
   if (missing(ix) & incremental==TRUE) {
     stop('Found incremental=TRUE but missing ix.')
   }
   
   if (!missing(ix)) {
-    p <- hommel@p[ix]
-    k <- length(p)
+    k <- length(hommel@p[ix])
   }
 
-  if (any(is.na(p)))
+  if (any(is.na(hommel@p[ix])))
     stop('NAs produced by selecting with ix.')
 
   if (k == 0) {
@@ -27,8 +26,13 @@ discoveries <- function(hommel, ix, incremental=FALSE, alpha=0.05)
   simesfactor <- hommel@simesfactor[h+1]
 
   allsortedp <- hommel@p[hommel@sorter]
+  
+  ix_sortedp <- integer(m)
+  names(ix_sortedp) <- names(hommel@p)
+  ix_sortedp[hommel@sorter] <- 1:m
+  ix_sortedp <- ix_sortedp[ix]
 
-  discoveries <- findDiscoveries(p, allsortedp, simesfactor, h, alpha, k, m)
+  discoveries <- findDiscoveries(ix_sortedp, allsortedp, simesfactor, h, alpha, k, m)
 
   if(!incremental) 
     return(discoveries[k+1])
@@ -42,11 +46,10 @@ tdp <- function(hommel, ix, incremental=FALSE, alpha=0.05)
 {
   m <- length(hommel@p)
   if (missing(ix)) {
-    d <- discoveries(hommel, incremental=incremental, alpha=alpha)
     k <- m
+    d <- discoveries(hommel, incremental=incremental, alpha=alpha)
   } else {
-    p <- hommel@p[ix]
-    k <- length(p)
+    k <- length(hommel@p[ix])
     d <- discoveries(hommel, ix, incremental=incremental, alpha=alpha)
   }
   d/k
